@@ -12,7 +12,7 @@
 //! This allows you to get the username, password verifier and salt values.
 //! The [`SrpVerifier`] page contains specifics.
 //! The [`SrpVerifier`] struct is created using [`NormalizedString`]s, the reasoning is
-//! explained on the [`NormalizedString`] page.
+//! explained on the [`normalized_string`](`crate::normalized_string`) page.
 //!
 //! This could look something like this:
 //! ```
@@ -39,10 +39,9 @@
 //! }
 //!
 //! fn save_values_to_database(username: &str,
-//!                            password_verifier: &[u8; PASSWORD_VERIFIER_LENGTH],
-//!                            salt: &[u8; SALT_LENGTH]) {
+//!                            password_verifier: &[u8; PASSWORD_VERIFIER_LENGTH as usize],
+//!                            salt: &[u8; SALT_LENGTH as usize]) {
 //!     // DB specific stuff here.
-//!     // In real life database could fail
 //! }
 //! ```
 //!
@@ -52,15 +51,18 @@
 //! This text focuses on a high level overview and important notices.
 //!
 //! The [Typestate](https://yoric.github.io/post/rust-typestate/) pattern is used
-//! in order to prevent accidental incorrect use.
+//! in order to prevent incorrect use.
 //! This means that whenever the next step of computation takes place, you call a function
 //! taking `self`, consuming the old object, and returning the new object.
 //!
 //! When authenticating players attempting to connect to the server, we start from where we let go
 //! with the last example.
-//! First, an [`SrpVerifier`] is created using the database values,
-//! then [`SrpVerifier::into_proof`] is called to convert it into an [`SrpProof`],
-//! finally [`SrpProof::into_server`] is called to convert it into an [`SrpServer`].
+//!
+//! * First an [`SrpVerifier`] is created using the database values,
+//! * Then [`SrpVerifier::into_proof`] is called to convert it into an [`SrpProof`],
+//! * Finally [`SrpProof::into_server`] is called to convert it into an [`SrpServer`] and a
+//! server proof.
+//!
 //! The [`SrpServer`] means that the client has been correctly authenticated
 //! and can be sent the realmlist.
 //! The [`SrpServer`] also provides the possibility of authenticating reconnects.
@@ -71,8 +73,8 @@
 //! The state machine goes like this:
 //! ```text
 //! SrpVerifier -> SrpProof -> mut SrpServer -|
-//!                                    ^      |
-//!                                    |      |
+//!                   +                ^      |
+//!              server_proof          |      |
 //!                                    |------|
 //! ```
 //! Where you would keep a mutable [`SrpServer`] around for reconnects and using the session key.
@@ -92,7 +94,7 @@
 //!     let password_verifier = [ 106, 6, 11, 113, 103, 55, 49, 130, 210, 249, 178, 176, 73, 77, 229, 163, 127, 223, 122, 163, 245, 174, 60, 217, 151, 142, 169, 173, 208, 8, 152, 31, ];
 //!     let salt = [ 120, 156, 208, 137, 73, 108, 21, 91, 28, 22, 13, 255, 99, 116, 71, 102, 158, 70, 65, 189, 153, 244, 143, 13, 214, 200, 160, 94, 217, 112, 206, 125, ];
 //!
-//!     let verifier = SrpVerifier::from_database_values(username, &password_verifier, &salt);
+//!     let verifier = SrpVerifier::from_database_values(username, password_verifier, salt);
 //!     let proof = verifier.into_proof();
 //!
 //!     // Gotten from client
@@ -104,11 +106,11 @@
 //!
 //!     // Can fail on proof comparison which means the password is incorrect.
 //!     // Send a failure packet and drop the connection.
-//!     let mut server = proof.into_server(client_public_key, &client_proof)?;
+//!     let (mut server, server_proof) = proof.into_server(client_public_key, client_proof)?;
 //!     // If this passes the client is successfully authenticated.
 //!
 //!     // Send the proof to client to prove that the server also knows the correct password.
-//!     let server_proof = server.server_proof();
+//!
 //!     // Add the server object to e.g. a HashMap of authenticated users.
 //!     // Client will now send a 'Send Realmlist' packet
 //!
@@ -121,7 +123,7 @@
 //!     let client_proof = [ 37, 167, 41, 153, 253, 156, 41, 174, 225, 125, 246, 158, 106, 248, 158, 232, 146, 8, 242, 164, ];
 //!
 //!     // Returns true if the proofs match, client is allowed to reconnect
-//!     assert!(server.verify_reconnection_attempt(&client_data, &client_proof));
+//!     assert!(server.verify_reconnection_attempt(client_data, client_proof));
 //!
 //!     Ok(())
 //! }
@@ -182,10 +184,10 @@ use crate::{error::InvalidPublicKeyError, srp_internal};
 /// fn from_database() -> Result<(), NormalizedStringError> {
 ///     // Get these from the database
 ///     let username = NormalizedString::new("Alice")?;
-///     let password_verifier = [0u8; PASSWORD_VERIFIER_LENGTH];
-///     let salt = [0u8; SALT_LENGTH];
+///     let password_verifier = [0u8; PASSWORD_VERIFIER_LENGTH as usize];
+///     let salt = [0u8; SALT_LENGTH as usize];
 ///
-///     let verifier = SrpVerifier::from_database_values(username, &password_verifier, &salt);
+///     let verifier = SrpVerifier::from_database_values(username, password_verifier, salt);
 ///
 ///     // Next step is continuing into the state machine, see into_proof() and SrpProof for more.
 ///     let proof = verifier.into_proof();
@@ -205,10 +207,10 @@ pub struct SrpVerifier {
 
 impl SrpVerifier {
     #[doc(alias = "U")]
-    /// The [`NormalizedString`] representation of the username, see that for more details.
+    /// The [`normalized_string`](`crate::normalized_string`) representation of the username,
+    /// see that for more details.
     ///
     /// Called `U` and `<username>` in [RFC2945](https://tools.ietf.org/html/rfc2945).
-    /// Specific constraints can be found at [`NormalizedString`].
     pub fn username(&self) -> &str {
         self.username.as_ref()
     }
@@ -218,10 +220,10 @@ impl SrpVerifier {
     /// Array is **little endian**.
     ///
     /// Called `v` and `<password verifier>` in [RFC2945](https://tools.ietf.org/html/rfc2945).
-    /// Always at most [32 bytes (256 bits)](crate::PASSWORD_VERIFIER_LENGTH) in length
+    /// Always [32 bytes (256 bits)](crate::PASSWORD_VERIFIER_LENGTH) in length
     /// since the value is generated through
     /// the remainder of a [32 byte value](crate::LARGE_SAFE_PRIME_LENGTH).
-    pub const fn password_verifier(&self) -> &[u8; PASSWORD_VERIFIER_LENGTH] {
+    pub const fn password_verifier(&self) -> &[u8; PASSWORD_VERIFIER_LENGTH as usize] {
         self.password_verifier.as_le()
     }
 
@@ -232,11 +234,11 @@ impl SrpVerifier {
     /// Called `s`, `<salt from passwd file>` and `<salt>` in [RFC2945](https://tools.ietf.org/html/rfc2945).
     /// Always [32 bytes (256 bits)](crate::SALT_LENGTH) in length since the packet sent to
     /// the client has a fixed width.
-    pub const fn salt(&self) -> &[u8; SALT_LENGTH] {
+    pub const fn salt(&self) -> &[u8; SALT_LENGTH as usize] {
         self.salt.as_le()
     }
 
-    /// See [`NormalizedString`] for more information on the format.
+    /// See [`normalized_string`](`crate::normalized_string`) for more information on the format.
     /// Only use this for generating verifiers and salts to save to the database.
     /// Never use this by saving raw usernames and passwords on the database.
     pub fn from_username_and_password(
@@ -248,12 +250,12 @@ impl SrpVerifier {
         Self::with_specific_salt(username, password, &salt)
     }
 
-    /// See [`NormalizedString`] for more information on the format.
+    /// See [`normalized_string`](`crate::normalized_string`) for more information on the string format.
     /// Both arrays are **little endian**.
-    pub fn from_database_values(
+    pub const fn from_database_values(
         username: NormalizedString,
-        password_verifier: &[u8; PASSWORD_VERIFIER_LENGTH],
-        salt: &[u8; SALT_LENGTH],
+        password_verifier: [u8; PASSWORD_VERIFIER_LENGTH as usize],
+        salt: [u8; SALT_LENGTH as usize],
     ) -> Self {
         Self {
             username,
@@ -269,11 +271,14 @@ impl SrpVerifier {
     /// * Panics if the RNG returns an error. If RNG does not work the authentication server
     /// should not continue functioning and therefore panics.
     /// * _Very_ rarely panic if the server generated public key is invalid.
-    /// This is so unlikely that it is many times more likely that something has been compromised
-    /// so that the program can not safely continue operations.
-    /// Specifically the public key is generated from
-    /// `(password_verifier + 7^private_key) % large_safe_prime`
-    /// and it is invalid if either `public_key == 0` or `public_key % large_safe_prime == 0`.
+    ///
+    /// There are only two invalid states for the randomly generated server public key:
+    /// * All zeros.
+    /// * Exactly the same as [the large safe prime](`crate::LARGE_SAFE_PRIME_LITTLE_ENDIAN`).
+    ///
+    /// This is 2 out of `2^256` possible states. The chances of this occurring naturally are very slim.
+    /// It is significantly more likely that the RNG of the system has been compromised in which case
+    /// authentication is not possible.
     #[doc(alias = "M")]
     #[doc(alias = "M1")]
     #[doc(alias = "M2")]
@@ -281,8 +286,7 @@ impl SrpVerifier {
         let server_private_key = PrivateKey::randomized();
 
         Self::with_specific_private_key(self, server_private_key)
-            // TODO: Add please report this to github at ...
-            .expect("The generated public key was invalid. This is insanely unlikely and even if you only see this error once you should probably check that your random number generation has not been compromised in some way. See documentation for SrpVerifier.")
+            .expect("The generated public key was invalid. This is insanely unlikely and even if you only see this error once you should probably check that your random number generation has not been compromised in some way. See documentation for SrpVerifier. Please report this on Github at 'https://github.com/gtker/wow_srp'.")
     }
 
     fn with_specific_salt(
@@ -293,7 +297,7 @@ impl SrpVerifier {
         let password_verifier =
             srp_internal::calculate_password_verifier(&username, &password, salt);
 
-        Self::from_database_values(username, &password_verifier, salt.as_le())
+        Self::from_database_values(username, password_verifier, *salt.as_le())
     }
 
     fn with_specific_private_key(
@@ -350,7 +354,7 @@ impl SrpVerifier {
 /// let large_safe_prime = LARGE_SAFE_PRIME_LITTLE_ENDIAN;
 /// let generator = GENERATOR;
 /// #
-/// # let client_public_key = [255u8; PUBLIC_KEY_LENGTH];
+/// # let client_public_key = [255u8; PUBLIC_KEY_LENGTH as usize];
 ///
 /// // Public key gotten from client
 /// let client_public_key = PublicKey::from_le_bytes(&client_public_key);
@@ -362,8 +366,8 @@ impl SrpVerifier {
 /// };
 ///
 /// // Proof gotten from client
-/// let client_proof = [255u8; PROOF_LENGTH];
-/// let server = proof.into_server(client_public_key, &client_proof);
+/// let client_proof = [255u8; PROOF_LENGTH as usize];
+/// let server = proof.into_server(client_public_key, client_proof);
 /// let server = match server {
 ///     Ok(s) => {s}
 ///     Err(_) => {
@@ -398,7 +402,7 @@ impl SrpProof {
     /// Called `B` in [RFC2945](https://tools.ietf.org/html/rfc2945).
     /// Always [32 bytes (256 bits)](crate::PUBLIC_KEY_LENGTH) in length since the packet sent to the client has a fixed width.
     #[doc(alias = "B")]
-    pub const fn server_public_key(&self) -> &[u8; PUBLIC_KEY_LENGTH] {
+    pub const fn server_public_key(&self) -> &[u8; PUBLIC_KEY_LENGTH as usize] {
         self.server_public_key.as_le()
     }
 
@@ -408,13 +412,24 @@ impl SrpProof {
     /// Called `s`, `<salt from passwd file>` or `<salt>` in [RFC2945](https://tools.ietf.org/html/rfc2945).
     /// Always [32 bytes (256 bits)](crate::SALT_LENGTH) in length since the packet sent to the client has a fixed width.
     #[doc(alias = "s")]
-    pub const fn salt(&self) -> &[u8; SALT_LENGTH] {
+    pub const fn salt(&self) -> &[u8; SALT_LENGTH as usize] {
         self.salt.as_le()
     }
 
-    /// Converts to an [`SrpServer`] by using the client supplied public key and proof,
+    /// Converts to an [`SrpServer`] and server proof by using the client supplied public key and proof,
     /// consuming the [`SrpProof`].
     ///
+    /// The server proof value must be sent to the client in order to prove that the server knows the
+    /// same password as the client.
+    /// The server proof is called `M` in [RFC2945](https://tools.ietf.org/html/rfc2945),
+    /// and `M2` in other litterature.
+    /// This is different from the paramter to [`SrpProof::into_server`] which is referred to as `M1`
+    /// in other litterature, but both are referred to as `M` in
+    /// [RFC2945](https://tools.ietf.org/html/rfc2945).
+    ///
+    /// The server proof is a **little endian** array.
+    /// The server proof is always [20 bytes (160 bits)](crate::PROOF_LENGTH) because it's a SHA-1 hash.
+    ///
     /// The [`PublicKey`] is used instead of an array in order to break the validation of the
     /// public key and the calculation of the proof into separate steps.
     /// An invalid [`PublicKey`] is more likely to be the result of the client deliberately sending
@@ -426,7 +441,7 @@ impl SrpProof {
     /// to the client _public_ key and not the private key.
     ///
     /// The client_proof is called `M` in [RFC2945](https://tools.ietf.org/html/rfc2945) and `M1` in other
-    /// literature. This is different from the server proof in [`SrpServer::server_proof`] which is often called
+    /// literature. This is different from the server proof returned from [`SrpProof::into_server`] which is often called
     /// `M2`, but is still referred to as `M` in [RFC2945](https://tools.ietf.org/html/rfc2945).
     /// The client_proof is always [20 bytes (160 bits)](PROOF_LENGTH) in length because it's a SHA-1 hash.
     ///
@@ -442,7 +457,7 @@ impl SrpProof {
     /// #
     /// #    let proof = verifier.into_proof();
     /// #
-    /// # let client_public_key = [255u8; PUBLIC_KEY_LENGTH];
+    /// # let client_public_key = [255u8; PUBLIC_KEY_LENGTH as usize];
     /// // This will panic because the public key, username, passwords and proofs are not related
     /// // Public key gotten from client
     /// let client_public_key = PublicKey::from_le_bytes(&client_public_key);
@@ -453,9 +468,9 @@ impl SrpProof {
     ///     }
     /// };
     /// // Proof gotten from client
-    /// let client_proof = [255u8; PROOF_LENGTH];
+    /// let client_proof = [255u8; PROOF_LENGTH as usize];
     ///
-    /// let server = proof.into_server(client_public_key, &client_proof);
+    /// let server = proof.into_server(client_public_key, client_proof);
     /// let server = match server {
     ///     Ok(s) => {s}
     ///     Err(_) => {
@@ -469,8 +484,8 @@ impl SrpProof {
     pub fn into_server(
         self,
         client_public_key: PublicKey,
-        client_proof: &[u8; PROOF_LENGTH],
-    ) -> Result<SrpServer, MatchProofsError> {
+        client_proof: [u8; PROOF_LENGTH as usize],
+    ) -> Result<(SrpServer, [u8; PROOF_LENGTH as usize]), MatchProofsError> {
         let session_key = srp_internal::calculate_session_key(
             &client_public_key,
             &self.server_public_key,
@@ -502,16 +517,18 @@ impl SrpProof {
 
         let reconnect_challenge_data = ReconnectData::randomized();
 
-        Ok(SrpServer {
-            username: self.username,
-            server_proof,
-            session_key,
-            reconnect_challenge_data,
-        })
+        Ok((
+            SrpServer {
+                username: self.username,
+                session_key,
+                reconnect_challenge_data,
+            },
+            *server_proof.as_le(),
+        ))
     }
 }
 
-/// The final step of the server. Contains the session key, server proof, and reconnect logic.
+/// The final step of authentication. Contains the session key, and reconnect logic.
 ///
 /// This represents the final struct used in authentication, if this struct is constructed,
 /// the client is correctly authenticated and should be allowed to connect to the server.
@@ -520,7 +537,7 @@ impl SrpProof {
 ///
 /// This struct should be saved in session storage to allow for clients to reconnect.
 /// If a client disconnects and reconnects properly again, the old struct should be replaced with
-/// the new one.
+/// the new one because the session key will be different.
 ///
 /// Created from [`SrpProof::into_server`].
 ///
@@ -535,17 +552,16 @@ impl SrpProof {
 /// let username = "Alice";
 /// # let verifier = SrpVerifier::from_username_and_password(NormalizedString::new(username).unwrap(), NormalizedString::new("password123").unwrap());
 /// # let proof = verifier.into_proof();
-/// # let client_public_key = PublicKey::from_le_bytes(&[1u8; PUBLIC_KEY_LENGTH]).unwrap();
-/// # let client_proof = [1u8; PROOF_LENGTH];
+/// # let client_public_key = PublicKey::from_le_bytes(&[1u8; PUBLIC_KEY_LENGTH as usize]).unwrap();
+/// # let client_proof = [1u8; PROOF_LENGTH as usize];
 /// let mut authenticated_clients = HashMap::new();
 ///
 /// // Server is created from unseen elements
-/// // Pretend there's error handling here, it's hidden to not bloat up the example
-/// let mut server = proof.into_server(client_public_key, &client_proof);
-/// # let server =  match server {
-/// #     Ok(s) => {s}
-/// #     Err(_) => return,
-/// # };
+/// let mut server = proof.into_server(client_public_key, client_proof);
+///  let (server, server_proof) =  match server {
+///      Ok(s) => {s}
+///      Err(_) => return,
+///  };
 ///
 /// // Add the server to session storage, for example a HashMap
 /// authenticated_clients.insert(username, server);
@@ -565,11 +581,11 @@ impl SrpProof {
 /// let server_challenge_data = client.reconnect_challenge_data();
 ///
 /// // Get client data and proof in the response
-/// # let client_challenge_data = [1u8; RECONNECT_CHALLENGE_DATA_LENGTH];
-/// # let client_proof = [1u8; PROOF_LENGTH];
+/// # let client_challenge_data = [1u8; RECONNECT_CHALLENGE_DATA_LENGTH as usize];
+/// # let client_proof = [1u8; PROOF_LENGTH as usize];
 ///
-/// let should_allow_connection = client.verify_reconnection_attempt(&client_challenge_data,
-///                                                                  &client_proof);
+/// let should_allow_connection = client.verify_reconnection_attempt(client_challenge_data,
+///                                                                  client_proof);
 /// if should_allow_connection {
 ///     // The client has proven that it knows the session key,
 ///     // continue with the connection
@@ -581,27 +597,11 @@ impl SrpProof {
 /// ```
 pub struct SrpServer {
     username: NormalizedString,
-    server_proof: Proof,
     session_key: SessionKey,
     reconnect_challenge_data: ReconnectData,
 }
 
 impl SrpServer {
-    /// The server proof of knowing the password. Called `M` in
-    /// [RFC2945](https://tools.ietf.org/html/rfc2945) and `M2` in
-    /// other literature. This is different from the parameter for [`SrpProof::into_server`] which
-    /// is referred to as `M1` in literature, but both are just called `M` in
-    /// [RFC2945](https://tools.ietf.org/html/rfc2945).
-    ///
-    /// This value must be sent to the client in order to prove that the server also knows the correct
-    /// password.
-    ///
-    /// The proof is always [20 bytes (160 bits)](crate::PROOF_LENGTH) because it's a SHA-1 hash.
-    #[doc(alias = "M2")]
-    pub const fn server_proof(&self) -> &[u8; PROOF_LENGTH] {
-        self.server_proof.as_le()
-    }
-
     /// Called `S` in [RFC2945](https://tools.ietf.org/html/rfc2945) and sometimes `K` or `key`
     /// in other literature.
     ///
@@ -612,7 +612,7 @@ impl SrpServer {
     /// created from 2 SHA-1 hashes of [20 bytes (160 bits)](PROOF_LENGTH).
     #[doc(alias = "K")]
     #[doc(alias = "S")]
-    pub const fn session_key(&self) -> &[u8; SESSION_KEY_LENGTH] {
+    pub const fn session_key(&self) -> &[u8; SESSION_KEY_LENGTH as usize] {
         self.session_key.as_le()
     }
 
@@ -626,7 +626,9 @@ impl SrpServer {
     /// Not mentioned in [RFC2945](https://tools.ietf.org/html/rfc2945) at all.
     ///
     /// See [`verify_reconnection_attempt`](SrpServer::verify_reconnection_attempt) for more.
-    pub const fn reconnect_challenge_data(&self) -> &[u8; RECONNECT_CHALLENGE_DATA_LENGTH] {
+    pub const fn reconnect_challenge_data(
+        &self,
+    ) -> &[u8; RECONNECT_CHALLENGE_DATA_LENGTH as usize] {
         self.reconnect_challenge_data.as_le()
     }
 
@@ -649,8 +651,8 @@ impl SrpServer {
     /// Not mentioned in [RFC2945](https://tools.ietf.org/html/rfc2945) at all.
     pub fn verify_reconnection_attempt(
         &mut self,
-        client_data: &[u8; RECONNECT_CHALLENGE_DATA_LENGTH],
-        client_proof: &[u8; PROOF_LENGTH],
+        client_data: [u8; RECONNECT_CHALLENGE_DATA_LENGTH as usize],
+        client_proof: [u8; PROOF_LENGTH as usize],
     ) -> bool {
         let server_proof = calculate_reconnect_proof(
             &self.username,
@@ -659,7 +661,7 @@ impl SrpServer {
             &self.session_key,
         );
 
-        let client_proof = Proof::from_le_bytes(&client_proof);
+        let client_proof = Proof::from_le_bytes(client_proof);
 
         let reconnect_verified = server_proof == client_proof;
 
@@ -712,10 +714,10 @@ mod test {
         .unwrap();
         let client_proof = Proof::from_be_hex_str("b91e6e0c8c06969c44585d9f66d73454f60a43e6");
 
-        let s = s
-            .into_server(client_public_key, &client_proof.as_le())
+        let (_s, server_proof) = s
+            .into_server(client_public_key, *client_proof.as_le())
             .unwrap();
-        let server_proof = Proof::from_le_bytes(&s.server_proof());
+        let server_proof = Proof::from_le_bytes(server_proof);
 
         assert_eq!(
             server_proof,
