@@ -20,20 +20,10 @@ use crate::primes::{Generator, KValue, LargeSafePrime};
 
 /// Only used for the [`calculate_client_proof`] function. Since the large safe prime and generator are
 /// statically determined we can precalculate it. See also the [`calculate_xor_hash`] function.
-const PRECALCULATED_XOR_HASH: [u8; SHA1_HASH_LENGTH] = [
+const PRECALCULATED_XOR_HASH: [u8; SHA1_HASH_LENGTH as usize] = [
     221, 123, 176, 58, 56, 172, 115, 17, 3, 152, 124, 90, 80, 111, 202, 150, 108, 123, 194, 167,
 ];
 
-#[doc(hidden)]
-#[macro_export]
-macro_rules! pad_little_endian_vec_to_array {
-    ($name: ident; $size: expr) => {{
-        let mut c = [0u8; $size];
-        c[0..$name.len()].clone_from_slice(&$name);
-        c
-    }};
-}
-
 /// Calculate the `x` value which is used for generating the password verifier `v`. See [calculate_password_verifier].
 ///
 /// `x` is calculated as `H( salt | H( upper( username | : |  password ) ) )` as described on page 3 of [RFC2945] and page 8 of [RFC5054].
@@ -73,7 +63,7 @@ pub fn calculate_x(
 
     let x = Sha1::new().chain(salt.as_le()).chain(p).finalize();
 
-    Sha1Hash::from_le_bytes(&x.into())
+    Sha1Hash::from_le_bytes(x.into())
 }
 
 /// Calculate the password verifier `v` used for generating the server public key `B` and the session key intermediate value `S`.
@@ -104,15 +94,15 @@ pub fn calculate_password_verifier(
     password: &NormalizedString,
     salt: &Salt,
     // Return an array instead of Verifier because this is never directly used to create a Verifier
-) -> [u8; PASSWORD_VERIFIER_LENGTH] {
+) -> [u8; PASSWORD_VERIFIER_LENGTH as usize] {
     let x = calculate_x(username, password, &salt).to_bigint();
 
     let generator = Generator::default().to_bigint();
     let large_safe_prime = LargeSafePrime::default().to_bigint();
 
-    let password_verifier = generator.modpow(&x, &large_safe_prime).to_bytes_le().1;
+    let password_verifier = generator.modpow(&x, &large_safe_prime);
 
-    pad_little_endian_vec_to_array!(password_verifier; PASSWORD_VERIFIER_LENGTH)
+    password_verifier.to_padded_32_byte_array_le()
 }
 
 /// Calculate the server public key `B`.
@@ -136,7 +126,7 @@ pub fn calculate_u(client_public_key: &PublicKey, server_public_key: &PublicKey)
         .chain(client_public_key.as_le())
         .chain(server_public_key.as_le())
         .finalize();
-    Sha1Hash::from_le_bytes(&s.into())
+    Sha1Hash::from_le_bytes(s.into())
 }
 
 /// Calculate the `S` value used for generating the session key.
@@ -163,26 +153,26 @@ pub fn calculate_S(
 pub fn calculate_interleaved(S: &SKey) -> SessionKey {
     let S = S.to_equal_slice();
 
-    let mut E = [0u8; S_LENGTH / 2];
+    let mut E = [0u8; (S_LENGTH / 2) as usize];
     for (i, e) in S.iter().step_by(2).enumerate() {
         E[i] = *e;
     }
     let G = Sha1::new().chain(&E[..S.len() / 2]).finalize();
 
-    let mut F = [0u8; S_LENGTH / 2];
+    let mut F = [0u8; (S_LENGTH / 2) as usize];
     for (i, f) in S.iter().skip(1).step_by(2).enumerate() {
         F[i] = *f;
     }
     let H = Sha1::new().chain(&F[..S.len() / 2]).finalize();
 
-    let mut result = [0u8; SESSION_KEY_LENGTH];
+    let mut result = [0u8; SESSION_KEY_LENGTH as usize];
     let zip = G.iter().zip(H.iter());
     for (i, r) in zip.enumerate() {
         result[i * 2] = *r.0;
         result[(i * 2) + 1] = *r.1;
     }
 
-    SessionKey::from_le_bytes(&result)
+    SessionKey::from_le_bytes(result)
 }
 
 // Returns a 40 byte big endian array.
@@ -215,7 +205,7 @@ pub fn calculate_server_proof(
         .chain(session_key.as_le())
         .finalize();
 
-    Proof::from_le_bytes(&s.into())
+    Proof::from_le_bytes(s.into())
 }
 
 pub(crate) fn calculate_xor_hash(
@@ -226,12 +216,12 @@ pub(crate) fn calculate_xor_hash(
 
     let g_hash = Sha1::new().chain([generator.as_u8()]).finalize();
 
-    let mut xor_hash = [0u8; SHA1_HASH_LENGTH];
+    let mut xor_hash = [0u8; SHA1_HASH_LENGTH as usize];
     for (i, n) in large_safe_prime_hash.iter().enumerate() {
         xor_hash[i] = *n as u8 ^ g_hash[i];
     }
 
-    Sha1Hash::from_le_bytes(&xor_hash)
+    Sha1Hash::from_le_bytes(xor_hash)
 }
 
 pub fn calculate_client_proof(
@@ -243,7 +233,7 @@ pub fn calculate_client_proof(
 ) -> Proof {
     let username_hash = Sha1::new().chain(username.as_ref()).finalize();
 
-    let out: [u8; PROOF_LENGTH] = Sha1::new()
+    let out: [u8; PROOF_LENGTH as usize] = Sha1::new()
         .chain(PRECALCULATED_XOR_HASH)
         .chain(username_hash)
         .chain(salt.as_le())
@@ -253,7 +243,7 @@ pub fn calculate_client_proof(
         .finalize()
         .into();
 
-    Proof::from_le_bytes(&out)
+    Proof::from_le_bytes(out)
 }
 
 pub fn calculate_reconnect_proof(
@@ -269,7 +259,7 @@ pub fn calculate_reconnect_proof(
         .chain(session_key.as_le())
         .finalize();
 
-    Proof::from_le_bytes(&s.into())
+    Proof::from_le_bytes(s.into())
 }
 
 #[cfg(test)]
@@ -373,7 +363,7 @@ mod test {
 
                 let expected = Verifier::from_be_hex_str(line.next().unwrap());
 
-                let v = Verifier::from_le_bytes(&calculate_password_verifier(
+                let v = Verifier::from_le_bytes(calculate_password_verifier(
                     &username, &password, &salt,
                 ));
 
